local bump = require('bump')

local player = {
    isPlayer = true,
    typ = "player",
    x = 51,
    y = 50,
    w = 16, 
    h = 16,
    vx = -20, 
    vy = 0,
    rvx = 0,
    speed = 30
}

local block = {
    typ = "block",
    x = 50,
    y = 150,
    w = 200, 
    h = 16
}

local push = {
    typ = "push",
    x = 20,
    y = 20,
    w = 20, 
    h = 100,
    speedx = 40,
    speedy = 0
}

function love.load()
    world  = bump.newWorld(32)
    world:add(player, player.x, player.y, player.w, player.h)
    world:add(block, block.x, block.y, block.w, block.h)
    world:add(push, push.x, push.y, push.w, push.h)
end

function collidewithnormal(player, dx, dy)
  -- simply update player's location based on the push amount
  local x,y = world:getRect(player)
  x, y = x + dx, y + dy
  world:update ( player, x, y )
  player.x, player.y = x, y
end

local pushFilter = function(item, other) return 'cross' end
function updatepush(dt)
    local oldx, oldy = push.x, push.y
    
    local futurex = push.x + dt * push.speedx
    local futurey = push.y + dt * push.speedy
    
    -- use a filter that returns "cross" so we don't stop moving when touching player. Pusher wants to move all the way!
    local newx, newy, cols, len = world:check(push, futurex, futurey, pushFilter)
    
    local dx,dy
    for i = 1, len do
        local col = cols[i]
        local other = col.other
        if other.isPlayer then
            -- get the difference between touch and final position
            dx, dy = (newx - col.touch.x),
                     (newy - col.touch.y)
            collidewithnormal(other, dx, dy)
        end
    end
    push.x, push.y = newx, newy
    world:update(push, newx, newy)
    if (push.x + push.w) >= 100 or push.x <= 20 then
        push.speedx = -push.speedx
    end
end

local playerFilter = function(item, other) return 'slide' end
function updateplayer(dt)
    player.prevx, player.prevy = player.x, player.y

    print("v", player.vx, player.vy)
    local futurex = player.x + player.vx * dt
    local futurey = player.y + player.vy * dt
    print("player", player.x, futurex)
    
    -- make player "slide" against all other objects, effectively making all objects behave like walls for it, pusher or not
    local newx, newy, cols, len = world:move(player, futurex, futurey, playerFilter)
    
    local dx, dy = (newx- player.prevx), (newy- player.prevy)
    player.x, player.y = newx, newy
end

local ldt = 0
function love.update(dt)
    ldt = dt
    updatepush(dt)
    updateplayer(dt)
end

function love.draw()
    love.graphics.rectangle('line', player.x, player.y, player.w, player.h)
    love.graphics.rectangle('fill', block.x, block.y, block.w, block.h)
    love.graphics.rectangle('fill', push.x, push.y, push.w, push.h)
    love.graphics.print("fps: "..tostring(love.timer.getFPS( )), 10, 10)
    love.graphics.print("dt: "..tostring(ldt), 10, 25)
    love.graphics.print(string.format("%0.5f", player.vx) .. ":" .. string.format("%0.5f", player.vy), 10,40)
end

function love.keypressed(key)
    if key == "escape" then
        love.event.quit()
    end
end